<?php

namespace Drupal\Tests\panelizer\Kernel;

use Drupal\KernelTests\KernelTestBase;
use Drupal\layout_builder\Section;
use Drupal\panelizer\LayoutBuilderMigration;
use Drupal\panels\Plugin\DisplayVariant\PanelsDisplayVariant;
use Drupal\Tests\node\Traits\ContentTypeCreationTrait;

/**
 * @coversDefaultClass \Drupal\panelizer\LayoutBuilderMigration
 *
 * @group panelizer
 */
class LayoutBuilderMigrationTest extends KernelTestBase {

  use ContentTypeCreationTrait;

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'ctools',
    'ctools_block',
    'field',
    'field_ui',
    'layout_discovery',
    'node',
    'panelizer',
    'panels',
    'system',
    'text',
    'user',
  ];

  /**
   * {@inheritdoc}
   */
  protected function setUp() {
    parent::setUp();
    $this->installConfig('node');
    $this->createContentType(['type' => 'page']);

    $this->container->get('panelizer')
      ->setPanelizerSettings('node', 'page', 'default', [
        'enable' => TRUE,
        'allow' => TRUE,
        'custom' => TRUE,
        'default' => 'default',
      ]);
  }

  /**
   * @covers ::toSection
   */
  public function testToSection() {
    /** @var \Drupal\panels\Plugin\DisplayVariant\PanelsDisplayVariant $display */
    $display = $this->container->get('panelizer')
      ->getDefaultPanelsDisplay('default', 'node', 'page', 'default');

    $this->assertInstanceOf(PanelsDisplayVariant::class, $display);

    $layout = $display->getLayout();

    $configuration = $display->getConfiguration();
    $this->assertNotEmpty($configuration['blocks']);

    // Normally this would not be kosher, but in this case the method really
    // does deserve private static visibility and should be tested by using
    // reflection to pry it open. It is too important to the migration to NOT
    // have a dedicated test.
    $method = new \ReflectionMethod(LayoutBuilderMigration::class, 'toSection');
    $method->setAccessible(TRUE);

    $section = $method->invoke(NULL, $display, 'node', 'page');
    $this->assertInstanceOf(Section::class, $section);
    $this->assertSame($layout->getPluginId(), $section->getLayoutId());
    $this->assertSame($layout->getConfiguration(), $section->getLayoutSettings());

    foreach ($configuration['blocks'] as $uuid => $block) {
      $component = $section->getComponent($uuid);
      $this->assertSame($uuid, $component->getUuid());
      $this->assertSame($block['region'], $component->getRegion());
      $this->assertSame($block['weight'], $component->getWeight());

      if (strpos($block['id'], 'entity_field:') === 0) {
        list(, , $field_name) = explode(':', $block['id']);
        $this->assertSame("field_block:node:page:$field_name", $component->getPluginId());

        // If the 'entity' context is mapped to Panelizer's entity context,
        // assert that mapping has been deleted, since it's not necessary (and
        // in fact causes errors) with Layout Builder.
        if (isset($block['context_mapping']['entity']) && $block['context_mapping']['entity'] === '@panelizer.entity_context:entity') {
          $component_configuration = $component->get('configuration');
          $this->assertArrayNotHasKey('entity', $component_configuration['context_mapping']);
        }
      }
      else {
        $this->assertSame($block['id'], $component->getPluginId());
      }
    }
  }

}
